﻿using System.Collections.Generic;
using Microsoft.Xna.Framework.Graphics;
using Vertex = Microsoft.Xna.Framework.Graphics.VertexPositionNormalTexture;
using System;

namespace XELF.Subdivision {

	public class MeshConverter {
		readonly DesignMesh source;
		public List<Vertex> Vertices;
		public List<int> Indices;

		int AddVertex(Face face, int faceVertex) {
			int index = Vertices.Count;
			Vertex vertex;
			var designVertex = source.Vertices[face.Vertices[faceVertex].Vertex];
			vertex.Position = designVertex.Position;
			vertex.Normal = face.Vertices[faceVertex].Normal;
			vertex.TextureCoordinate = face.Vertices[faceVertex].TexCoord;
			Vertices.Add(vertex);

			return index;
		}

		public MeshConverter(DesignMesh source) {
			this.source = source;
			Vertices = new List<VertexPositionNormalTexture>();
			Indices = new List<int>();

			foreach (var face in source.Faces) {
				int a = AddVertex(face, 0);
				int b = AddVertex(face, 1);
				for (int v = 2; v < face.Vertices.Count; v++) {
					int c = AddVertex(face, v);
					Indices.Add(a);
					Indices.Add(b);
					Indices.Add(c);
					b = c;
				}
			}
		}

		public static Mesh Convert(GraphicsDevice graphicsDevice, DesignMesh source) {
			var converter = new MeshConverter(source);
			return new Mesh(graphicsDevice, converter.Vertices.ToArray(), converter.Indices.ToArray());
		}
	}

	public class Mesh : IDisposable {
		readonly GraphicsDevice graphicsDevice;
		readonly VertexDeclaration declaration;
		public VertexPositionNormalTexture[] Vertices;
		public int[] Indices;
		public BasicEffect Effect;

		public Mesh(GraphicsDevice graphicsDevice, Vertex[] vertices, int[] indices) {
			this.graphicsDevice = graphicsDevice;
			Vertices = vertices;
			Indices = indices;
			declaration = new VertexDeclaration(graphicsDevice, VertexPositionNormalTexture.VertexElements);
			Effect = new BasicEffect(graphicsDevice, null);
		}
		public static Mesh CreateFrom(GraphicsDevice graphicsDevice, DesignMesh source) {
			return MeshConverter.Convert(graphicsDevice, source);
		}

		public void Draw() {
			Effect.Begin();

			graphicsDevice.VertexDeclaration = declaration;

			foreach (var pass in Effect.CurrentTechnique.Passes) {
				pass.Begin();
				graphicsDevice.DrawUserIndexedPrimitives(PrimitiveType.TriangleList, Vertices,
					0, Vertices.Length, Indices, 0, Indices.Length / 3);
				pass.End();
			}
			Effect.End();
		}
		public void Dispose() {
			declaration.Dispose();
			Effect.Dispose();
		}
	}

}
