using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace XELF.Subdivision {

	public class SubdivisionGame : Microsoft.Xna.Framework.Game {
		readonly GraphicsDeviceManager graphics;
		SpriteBatch spriteBatch;
		SubdivisionService subdivision;
		Mesh mesh0;
		Mesh mesh1;
		int level = 2;
		bool solidVisible = true;
		bool wireframeVisible = true;
		bool pointVisible = true;
		KeyboardState previousState;
		KeyboardState currentState;
		Texture2D texture;

		public SubdivisionGame() {
			graphics = new GraphicsDeviceManager(this);
			graphics.PreferMultiSampling = true;
			Content.RootDirectory = "Content";
		}

		protected override void LoadContent() {
			spriteBatch = new SpriteBatch(GraphicsDevice);
			texture = new Texture2D(GraphicsDevice, 1, 1);
			texture.SetData(new Color[] { Color.White });

			DesignMesh source = new DesignMesh(
				new List<DesignVertex> {
					new DesignVertex{ Position = new Vector3(0,0,0), },
					new DesignVertex{ Position = new Vector3(1,0,0), },
					new DesignVertex{ Position = new Vector3(1,1,0), },
					new DesignVertex{ Position = new Vector3(0,1,0), },
					new DesignVertex{ Position = new Vector3(1,2,0), },
					new DesignVertex{ Position = new Vector3(0,2,0), },
					new DesignVertex{ Position = new Vector3(1,2,1), },
					new DesignVertex{ Position = new Vector3(0,2,1), },
					new DesignVertex{ Position = new Vector3(1,1,1), },
				},
				new List<Face> {
					new Face{ Vertices = new List<FaceVertex> {
						new FaceVertex { Vertex = 0, Normal = new Vector3(0,0,1), },
						new FaceVertex { Vertex = 1, Normal = new Vector3(0,0,1), },
						new FaceVertex { Vertex = 2, Normal = new Vector3(0,0,1), },
						new FaceVertex { Vertex = 3, Normal = new Vector3(0,0,1), },
					}},
					new Face{ Vertices = new List<FaceVertex> {
						new FaceVertex { Vertex = 3, Normal = Vector3.Normalize(new Vector3(0,0,1)), },
						new FaceVertex { Vertex = 2, Normal = Vector3.Normalize(new Vector3(0,0,1)), },
						new FaceVertex { Vertex = 4, Normal = Vector3.Normalize(new Vector3(1,1,1)), },
						new FaceVertex { Vertex = 5, Normal = Vector3.Normalize(new Vector3(1,0,1)), },
					}},
					new Face{ Vertices = new List<FaceVertex> {
						new FaceVertex { Vertex = 5, Normal = Vector3.Normalize(new Vector3(1,0,1)), },
						new FaceVertex { Vertex = 4, Normal = Vector3.Normalize(new Vector3(1,1,1)), },
						new FaceVertex { Vertex = 6, Normal = Vector3.Normalize(new Vector3(1,1,0)), },
						new FaceVertex { Vertex = 7, Normal = Vector3.Normalize(new Vector3(1,0,0)), },
					}},
					new Face{ Vertices = new List<FaceVertex> {
						new FaceVertex { Vertex = 2, Normal = Vector3.Normalize(new Vector3(0,0,1)), },
						new FaceVertex { Vertex = 4, Normal = Vector3.Normalize(new Vector3(1,1,1)), },
						new FaceVertex { Vertex = 6, Normal = Vector3.Normalize(new Vector3(1,1,0)), },
						new FaceVertex { Vertex = 8, Normal = Vector3.Normalize(new Vector3(0,1,0)), },
					}},
				}
			);
			subdivision = new SubdivisionService(source);
			mesh0 = Mesh.CreateFrom(GraphicsDevice, source);

			UpdateLevel();
		}
		protected override void UnloadContent() {
			mesh0.Dispose();
			mesh1.Dispose();
		}

		void UpdateLevel() {
			if (mesh1 != null) mesh1.Dispose();

			DesignMesh subMesh = subdivision[level];
			mesh1 = Mesh.CreateFrom(GraphicsDevice, subMesh);
		}

		protected override void Update(GameTime gameTime) {
			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
				this.Exit();

			previousState = currentState;
			currentState = Keyboard.GetState();

			if (IsKeyPress(Keys.A)) {
				level--;
				if (level < 0) level = 0;
				UpdateLevel();
			}
			if (IsKeyPress(Keys.D)) {
				level++;
				if (level >= 5) level = 5;
				UpdateLevel();
			}
			if (IsKeyPress(Keys.F)) {
				solidVisible = !solidVisible;
			}
			if (IsKeyPress(Keys.W)) {
				wireframeVisible = !wireframeVisible;
			}
			if (IsKeyPress(Keys.P)) {
				pointVisible = !pointVisible;
			}

			base.Update(gameTime);
		}

		bool IsKeyPress(Keys key) {
			return currentState.IsKeyDown(key) && previousState.IsKeyUp(key);
		}

		protected override void Draw(GameTime gameTime) {
			GraphicsDevice.Clear(Color.CornflowerBlue);

			GraphicsDevice.RenderState.CullMode = CullMode.None;
			GraphicsDevice.RenderState.PointSize = 5;

			Matrix world = Matrix.CreateRotationY((float)gameTime.TotalGameTime.TotalSeconds);
			Matrix view = Matrix.CreateLookAt(new Vector3(0, 0, 4), new Vector3(0, 1.0f, 0), Vector3.Up);
			Matrix projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
				GraphicsDevice.Viewport.AspectRatio, 1.0f, 1000.0f);

			mesh0.Effect.World = world;
			mesh0.Effect.View = view;
			mesh0.Effect.Projection = projection;

			mesh1.Effect.PreferPerPixelLighting = true;
			mesh1.Effect.TextureEnabled = true;
			mesh1.Effect.Texture = texture;
			mesh1.Effect.EnableDefaultLighting();
			mesh1.Effect.World = world;
			mesh1.Effect.View = view;
			mesh1.Effect.Projection = projection;

			if (solidVisible) {
				GraphicsDevice.RenderState.FillMode = FillMode.Solid;
				//mesh0.Draw();
				mesh1.Effect.DiffuseColor = new Vector3(1, 1, 1);
				mesh1.Draw();
			}
			if (wireframeVisible) {
				GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;
				mesh0.Effect.DiffuseColor = new Vector3(0, 1, 0);
				mesh0.Draw();
				mesh1.Effect.DiffuseColor = new Vector3(1, 0, 0);
				mesh1.Draw();
			}
			if (pointVisible) {
				GraphicsDevice.RenderState.FillMode = FillMode.Point;
				mesh0.Effect.DiffuseColor = new Vector3(0, 1, 0);
				mesh0.Draw();
				mesh1.Effect.DiffuseColor = new Vector3(1, 0, 0);
				mesh1.Draw();
			}

			base.Draw(gameTime);
		}
	}
}
